@@ -99,10 +99,15 @@ class ConditionProcessor:
         reaching_conditions = {}
         # recover the reaching condition for each node
         sorted_nodes = CFGUtils.quasi_topological_sort_nodes(_g)
+        terminating_nodes = [ ]
         for node in sorted_nodes:
             preds = _g.predecessors(node)
             reaching_condition = None
 
+            out_degree = _g.out_degree(node)
+            if out_degree == 0:
+                terminating_nodes.append(node)
+
             if node is region.head:
                 # the head is always reachable
                 reaching_condition = claripy.true
@@ -123,6 +128,22 @@ class ConditionProcessor:
             if reaching_condition is not None:
                 reaching_conditions[node] = self.simplify_condition(reaching_condition)
 
+        # My hypothesis to be proved: in any regioned graph, there must be a node with a 0 out-degree whose reaching
+        # conditions can be marked as True. In other words, if all 0 out-degree nodes have non-trivial reaching
+        # conditions, we can always pick one of them and change its reaching condition to True, without changing the
+        # semantics of the regioned graph.
+        if terminating_nodes and all(not reaching_conditions[node].is_true() for node in terminating_nodes
+                                     if node in reaching_conditions):
+            # pick the node with the greatest in-degree
+            terminating_nodes = sorted(terminating_nodes, key=_g.in_degree)
+            node_with_greatest_indegree = terminating_nodes[-1]
+            if _g.in_degree(node_with_greatest_indegree) > 1:
+                # forcing the in-degree to be greater than 1 allows us to skip the case blocks in switch-cases
+                # otherwise structurer will fail to structure the control flow
+                reaching_conditions[node_with_greatest_indegree] = claripy.true
+                l.warning("Marking node %r as trivially reachable. Disable this optimization in condition_processor.py "
+                          "if it leads to incorrect decompilation result.", node_with_greatest_indegree)
+
         self.reaching_conditions = reaching_conditions
 
     def recover_reaching_conditions_for_jumptables(self, region, jump_tables, edge_conditions):
@@ -397,6 +418,21 @@ class ConditionProcessor:
     # Expression conversion
     #
 
+    def _convert_extract(self, hi, lo, expr, annotations, memo=None):
+        # ailment does not support Extract. We translate Extract to Convert and shift.
+        if lo == 0:
+            try:
+                tag_annotation = next(iter(anno for anno in annotations if isinstance(anno, TagsAnnotation)))
+                tags = tag_annotation.tags
+            except StopIteration:
+                tags = {}
+            return ailment.Expr.Convert(None, expr.size(), hi + 1, False,
+                                        self.convert_claripy_bool_ast(expr, memo=memo),
+                                        **tags,
+                                        )
+
+        raise NotImplementedError("This case will be implemented once encountered.")
+
     def convert_claripy_bool_ast(self, cond, memo=None):
         """
         Convert recovered reaching conditions from claripy ASTs to ailment Expressions
@@ -467,6 +503,7 @@ class ConditionProcessor:
             'BVV': lambda cond_: ailment.Expr.Const(None, None, cond_.args[0], cond_.size()),
             'BoolV': lambda cond_: ailment.Expr.Const(None, None, True, 1) if cond_.args[0] is True
                                                                         else ailment.Expr.Const(None, None, False, 1),
+            'Extract': lambda cond_: self._convert_extract(*cond_.args, cond_.annotations, memo=memo),
         }
 
         if cond.op in _mapping:
@@ -477,7 +514,7 @@ class ConditionProcessor:
     def claripy_ast_from_ail_condition(self, condition) -> claripy.ast.Base:
 
         # Unpack a condition all the way to the leaves
-        if isinstance(condition, claripy.ast.Base):
+        if isinstance(condition, claripy.ast.Base):  # pylint:disable=isinstance-second-argument-not-valid-type
             return condition
 
         def _op_with_unified_size(op, conv, operand0, operand1):
@@ -546,7 +583,7 @@ class ConditionProcessor:
             if condition.bits == 1:
                 var = claripy.BoolV('ailtmp_%d' % condition.tmp_idx)
             else:
-                var = claripy.BVS('ailtmp_%d' % condition.tmp_idx, condition.bits)
+                var = claripy.BVS('ailtmp_%d' % condition.tmp_idx, condition.bits, explicit_name=True)
             self._condition_mapping[var.args[0]] = condition
             return var
 
