@@ -42,8 +42,14 @@ class Decompiler(Analysis):
                                               kb=self.kb,
                                               optimization_passes=self._optimization_passes,
                                               sp_tracker_track_memory=self._sp_tracker_track_memory,
+                                              cfg=self._cfg,
                                               **self.options_to_params(options_by_class['clinic'])
                                               )
+        self.clinic = clinic
+
+        if clinic.graph is None:
+            # the function is empty
+            return
 
         cond_proc = ConditionProcessor()
 
@@ -61,7 +67,6 @@ class Decompiler(Analysis):
                                                                 kb=self.kb,
                                                                 variable_kb=clinic.variable_kb)
 
-        self.clinic = clinic
         self.codegen = codegen
 
     def _set_global_variables(self):
