@@ -144,6 +144,7 @@ void State::stop(stop_t reason) {
 	stop_details.block_size = block_details.block_size;
 	switch (reason) {
 		case STOP_SYSCALL:
+		case STOP_SYMBOLIC_MEM_DEP_NOT_LIVE:
 			commit();
 			break;
 	}
@@ -226,7 +227,7 @@ void State::commit() {
 			block_details.register_values.emplace_back(block_start_reg_values.at(concrete_reg));
 		}
 		for (auto &symbolic_instr: block_details.symbolic_instrs) {
-			if (symbolic_instr.has_memory_dep) {
+			if (symbolic_instr.has_concrete_memory_dep) {
 				archived_memory_values.emplace_back(mem_reads_map.at(symbolic_instr.instr_addr).memory_values);
 				symbolic_instr.memory_values = &(archived_memory_values.back()[0]);
 				symbolic_instr.memory_values_count = archived_memory_values.back().size();
@@ -646,8 +647,13 @@ void State::compute_slice_of_instrs(address_t instr_addr, const instruction_tain
 	for (auto &dependency: instr_taint_entry.dependencies_to_save) {
 		if (dependency.entity_type == TAINT_ENTITY_REG) {
 			vex_reg_offset_t dependency_full_register_offset = get_full_register_offset(dependency.reg_offset);
+			auto dep_reg_slice_entry = reg_instr_slice.find(dependency_full_register_offset);
+			if (dep_reg_slice_entry == reg_instr_slice.end()) {
+				// We don't care about slice of this register because it's artificial, blacklisted or something else.
+				continue;
+			}
 			if (!is_symbolic_register(dependency_full_register_offset)) {
-				auto dep_reg_slice_instrs = reg_instr_slice.at(dependency_full_register_offset);
+				auto dep_reg_slice_instrs = dep_reg_slice_entry->second;
 				if (dep_reg_slice_instrs.size() == 0) {
 					// The register was not modified in this block by any preceding instruction
 					// and so it's value at start of block is a dependency of the block
@@ -1279,15 +1285,16 @@ bool State::is_symbolic_register(vex_reg_offset_t reg_offset) const {
 		}
 		return false;
 	}
+	reg_offset = get_full_register_offset(reg_offset);
 	// The register is not a CPU flag and so we check every byte of the register
-	for (int i = 0; i < reg_size_map.at(reg_offset); i++) {
+	for (auto i = 0; i < reg_size_map.at(reg_offset); i++) {
 		// If any of the register's bytes are symbolic, we deem the register to be symbolic
 		if (block_symbolic_registers.count(reg_offset + i) > 0) {
 			return true;
 		}
 	}
 	bool is_concrete = true;
-	for (int i = 0; i < reg_size_map.at(reg_offset); i++) {
+	for (auto i = 0; i < reg_size_map.at(reg_offset); i++) {
 		if (block_concrete_registers.count(reg_offset) == 0) {
 			is_concrete = false;
 			break;
@@ -1299,7 +1306,7 @@ bool State::is_symbolic_register(vex_reg_offset_t reg_offset) const {
 	}
 	// If we reach here, it means that the register is not marked symbolic or concrete in the block
 	// level taint status tracker. We check the state's symbolic register list.
-	for (int i = 0; i < reg_size_map.at(reg_offset); i++) {
+	for (auto i = 0; i < reg_size_map.at(reg_offset); i++) {
 		if (symbolic_registers.count(reg_offset + i) > 0) {
 			return true;
 		}
@@ -1439,11 +1446,8 @@ void State::propagate_taint_of_one_instr(address_t instr_addr, const instruction
 
 	is_instr_symbolic = false;
 	instr_details = compute_instr_details(instr_addr, instr_taint_entry);
-	if (instr_taint_entry.has_memory_read) {
-		auto mem_read_result = mem_reads_map.at(instr_addr);
-		if (mem_read_result.is_value_symbolic) {
-			is_instr_symbolic = true;
-		}
+	if (instr_details.has_symbolic_memory_dep) {
+		is_instr_symbolic = true;
 	}
 	for (auto &taint_data_entry: instr_taint_entry.taint_sink_src_map) {
 		taint_entity_t taint_sink = taint_data_entry.first;
@@ -1527,14 +1531,17 @@ instr_details_t State::compute_instr_details(address_t instr_addr, const instruc
 	if (instr_taint_entry.has_memory_read) {
 		auto mem_read_result = mem_reads_map.at(instr_addr);
 		if (!mem_read_result.is_value_symbolic) {
-			instr_details.has_memory_dep = true;
+			instr_details.has_concrete_memory_dep = true;
+			instr_details.has_symbolic_memory_dep = false;
 		}
 		else {
-			instr_details.has_memory_dep = false;
+			instr_details.has_concrete_memory_dep = false;
+			instr_details.has_symbolic_memory_dep = true;
 		}
 	}
 	else {
-		instr_details.has_memory_dep = false;
+		instr_details.has_concrete_memory_dep = false;
+		instr_details.has_symbolic_memory_dep = false;
 	}
 	return instr_details;
 }
@@ -1639,6 +1646,9 @@ void State::update_register_slice(address_t instr_addr, const instruction_taint_
 		if ((full_register_offset == arch_pc_reg_vex_offset()) || is_symbolic_register(full_register_offset)) {
 			continue;
 		}
+		if (reg_instr_slice.find(full_register_offset) == reg_instr_slice.end()) {
+			continue;
+		}
 		if (!reg_entry.second) {
 			reg_instr_slice.at(full_register_offset).clear();
 		}
@@ -1659,7 +1669,34 @@ bool State::is_block_next_target_symbolic() const {
 	return (block_next_target_taint_status != TAINT_STATUS_CONCRETE);
 }
 
-address_t State::get_instruction_pointer() {
+bool State::check_symbolic_stack_mem_dependencies_liveness() const {
+	// Stop concrete execution if a stack frame was deallocated and if any symbolic memory dependencies were present
+	// on that stack frame.
+	address_t curr_stack_top_addr = get_stack_pointer();
+	if (curr_stack_top_addr <= prev_stack_top_addr) {
+		// No change in stack frame and so no need to perform a liveness check.
+		// TODO: What is stack growth direction is different?
+		return true;
+	}
+	for (auto &block: blocks_with_symbolic_instrs) {
+		auto &block_symbolic_instrs = block_details.symbolic_instrs;
+		for (auto symbolic_instr = block_symbolic_instrs.rbegin(); symbolic_instr != block_symbolic_instrs.rend(); symbolic_instr++) {
+			if (symbolic_instr->has_symbolic_memory_dep) {
+				auto mem_read_entry = mem_reads_map.at(symbolic_instr->instr_addr);
+				for (auto &mem_value: mem_read_entry.memory_values) {
+					if ((curr_stack_top_addr > mem_value.address) && (mem_value.address > prev_stack_top_addr)) {
+						// A symbolic memory value that this symbolic instruction depends on is no longer on the stack
+						// and could be overwritten by future code. We stop concrete execution here to avoid that.
+						return false;
+					}
+				}
+			}
+		}
+	}
+	return true;
+}
+
+address_t State::get_instruction_pointer() const {
 	address_t out = 0;
 	unsigned int reg = arch_pc_reg();
 	if (reg == -1) {
@@ -1671,7 +1708,7 @@ address_t State::get_instruction_pointer() {
 	return out;
 }
 
-address_t State::get_stack_pointer() {
+address_t State::get_stack_pointer() const {
 	address_t out = 0;
 	unsigned int reg = arch_sp_reg();
 	if (reg == -1) {
@@ -1772,7 +1809,12 @@ static void hook_block(uc_engine *uc, uint64_t address, int32_t size, void *user
 		state->ignore_next_selfmod = true;
 		return;
 	}
+	if (!state->check_symbolic_stack_mem_dependencies_liveness()) {
+		state->stop(STOP_SYMBOLIC_MEM_DEP_NOT_LIVE);
+		return;
+	}
 	state->commit();
+	state->update_previous_stack_top();
 	state->step(address, size);
 
 	if (!state->stopped) {
